from cheshire_cat_api.api_client import ApiClient
from cheshire_cat_api.api import (
    MemoryApi, PluginsApi, RabbitHoleApi, StatusApi,
    SettingsEmbedderApi, SettingsGeneralApi, SettingsLargeLanguageModelApi, SettingsPromptApi)


class CatAPI:
    """"The class to instantiate the Cat API endpoints"""

    def __init__(self, client: ApiClient):
        self.memory = MemoryApi(client)
        self.plugins = PluginsApi(client)
        self.rabbit_hole = RabbitHoleApi(client)
        self.status = StatusApi(client)
        self.embedder = SettingsEmbedderApi(client)
        self.general = SettingsGeneralApi(client)
        self.llm = SettingsLargeLanguageModelApi(client)
        self.prompt = SettingsPromptApi(client)


class WebSocketSettings:
    """
    Class containing the WebSocket configuration options and variables used by ccat-api package
    """

    def __init__(self):
        # Websocket path to use to communicate with the CCat
        self.path = 'ws'
        # The maximum number of retries before calling on_failed
        self.retries = 3
        # The delay for reconnect, in milliseconds
        self.delay = 5000
        # The function to call after failing to reconnect
        self.on_failed = None


class Settings:
    """
    Class containing all the configuration options and variables used by ccat-api package
    """

    def __init__(self):
        # The URL to which connect to the Cat
        self.base_url = 'localhost'
        # The key to authenticate the Cat endpoints
        self.auth_key = ''
        # The port to which connect to the Cat
        self.port = 1865
        # Choose to either use the secure protocol or not
        self.secure = False
        # Timeout for the endpoints, in milliseconds
        self.timeout = 10000
        # Choose to either instantly initialize websocket and api client or not
        self.instant = True
        # WebSocket Settings
        self.ws = WebSocketSettings()



# coding: utf-8

# flake8: noqa

"""
    😸 Cheshire-Cat API

    Customizable AI architecture  # noqa: E501

    The version of the OpenAPI document: 0.0.5
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""


__version__ = "0.2.0"

from cheshire_cat_api.cat_client import CatClient
from cheshire_cat_api.utils import Settings, WebSocketSettings

# import apis into sdk package
from cheshire_cat_api.api.memory_api import MemoryApi
from cheshire_cat_api.api.plugins_api import PluginsApi
from cheshire_cat_api.api.rabbit_hole_api import RabbitHoleApi
from cheshire_cat_api.api.settings_embedder_api import SettingsEmbedderApi
from cheshire_cat_api.api.settings_general_api import SettingsGeneralApi
from cheshire_cat_api.api.settings_large_language_model_api import SettingsLargeLanguageModelApi
from cheshire_cat_api.api.settings_prompt_api import SettingsPromptApi
from cheshire_cat_api.api.status_api import StatusApi

# import ApiClient
from cheshire_cat_api.api_response import ApiResponse
from cheshire_cat_api.api_client import ApiClient
from cheshire_cat_api.configuration import Configuration
from cheshire_cat_api.exceptions import OpenApiException
from cheshire_cat_api.exceptions import ApiTypeError
from cheshire_cat_api.exceptions import ApiValueError
from cheshire_cat_api.exceptions import ApiKeyError
from cheshire_cat_api.exceptions import ApiAttributeError
from cheshire_cat_api.exceptions import ApiException

# import models into sdk package
from cheshire_cat_api.models.body_upload_url import BodyUploadUrl
from cheshire_cat_api.models.http_validation_error import HTTPValidationError
from cheshire_cat_api.models.location_inner import LocationInner
from cheshire_cat_api.models.setting_body import SettingBody
from cheshire_cat_api.models.validation_error import ValidationError



# coding: utf-8

# flake8: noqa
"""
    😸 Cheshire-Cat API

    Customizable AI architecture  # noqa: E501

    The version of the OpenAPI document: 0.0.5
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""


# import models into model package
from cheshire_cat_api.models.body_upload_url import BodyUploadUrl
from cheshire_cat_api.models.http_validation_error import HTTPValidationError
from cheshire_cat_api.models.location_inner import LocationInner
from cheshire_cat_api.models.setting_body import SettingBody
from cheshire_cat_api.models.validation_error import ValidationError



import json
import time
from typing import Dict, Callable
from websocket import WebSocketApp
from threading import Thread
from cheshire_cat_api.api_client import ApiClient
from cheshire_cat_api.configuration import Configuration
from cheshire_cat_api.utils import Settings, CatAPI


class CatClient:
    """
    The class to communicate with the Cheshire Cat AI
    """

    def __init__(self,
                 settings: Settings | None = None,
                 on_open: Callable | None = None,
                 on_close: Callable | None = None,
                 on_message: Callable | None = None,
                 on_error: Callable | None = None
                 ):
        # Instantiate user message handlers if any, otherwise use default
        self.on_message = on_message
        self.on_error = on_error
        self.on_close = on_close
        self.on_open = on_open
        # WebSocket settings
        self.settings = settings if settings is not None else Settings()
        # Connection status
        self.is_closed = False
        self.is_started = False
        # Closed by the user
        self.explicitly_closed = False
        # Number of retries already done
        self.retried = 0
        # Start instantly
        if self.settings.instant:
            self.run()

    @property
    def url(self):
        """"Compose the WebSocket and API address"""
        secure = 's://' if self.settings.secure else '://'
        port = f":{self.settings.port}" if self.settings.port else ''
        return f"{secure}{self.settings.base_url}{port}"

    def _connect(self):
        """"Connect to the WebSocket in a separate thread"""
        self._ws = WebSocketApp(
            f"ws{self.url}/{self.settings.ws.path}",
            on_message=self.on_ws_message,
            on_error=self.on_ws_error,
            on_close=self.on_ws_close,
            on_open=self.on_ws_open
        )

        self.conn = Thread(target=self._ws.run_forever)
        self.conn.start()

    def run(self):
        """"Instantiate the WebSocket and Cat API client with configuration"""
        if self.is_started:
            raise Exception("The Cheshire Cat Client was already initialized")
        else:
            self._connect()
            config = Configuration()
            config.host = f'http{self.url}'
            self.api = CatAPI(ApiClient(
                configuration=config,
                header_name='access_token',
                header_value=self.settings.auth_key
            ))
            self.is_started = True

    def on_ws_open(self, ws):
        """"Default message handler on conneciton opening"""

        # Run user custom function
        if callable(self.on_open):
            self.on_open()
            return

        print("Connection established")

    def on_ws_message(self, ws, message: str):
        """"Default message handler when receiving a message"""
        # Run user custom function
        if callable(self.on_message):
            self.on_message(message)
            return

        answer = json.loads(message)
        print(answer["content"])

    def on_ws_error(self, ws, error: str):
        """"Default message handler on WebSocket error"""
        # Run user custom function
        if callable(self.on_error):
            self.on_error(error)
            return

        print(f"Encountered error: {error}")

    def on_ws_close(self, ws, close_status_code: int, msg: str):
        """"Default message handler on closed connection"""

        # Retry to connect if connection fails
        if not self.explicitly_closed:
            self.retried += 1
            if self.settings.ws.retries < 0 or self.retried < self.settings.ws.retries:
                self._connect()
                time.sleep(self.settings.ws.delay / 1000)
            else:
                self.settings.ws.on_failed()
            return

        self.is_closed = True if close_status_code is not None else False

        # Run user custom function
        if callable(self.on_close):
            self.on_close(close_status_code, msg)
            return

        print(f"Connection closed: {msg}")

    def send(self, message: str, prompt_settings: Dict = {}):
        """Send a message to WebSocket server using a separate thread"""
        if not self.is_closed:
            self._ws.send(json.dumps({
                "text": message,
                "prompt_settings": prompt_settings
            }))

    def close(self):
        # Close connection
        self.explicitly_closed = True
        self._ws.close()



# coding: utf-8

"""
    😸 Cheshire-Cat API

    Customizable AI architecture  # noqa: E501

    The version of the OpenAPI document: 0.0.5
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""

from __future__ import annotations
import pprint
import re  # noqa: F401
import json

from typing import Optional
from pydantic import BaseModel, Field, StrictBool, StrictInt, StrictStr


class BodyUploadUrl(BaseModel):
    """
    BodyUploadUrl
    """
    url: StrictStr = Field(..., description="URL of the website to which you want to save the content")
    chunk_size: Optional[StrictInt] = Field(400,
                                            description="Maximum length of each chunk after the document is split (in characters)")
    chunk_overlap: Optional[StrictInt] = Field(100, description="Chunk overlap (in characters)")
    summary: Optional[StrictBool] = Field(False, description="Enables call to summary hook for this website")
    __properties = ["url", "chunk_size", "chunk_overlap", "summary"]

    class Config:
        """Pydantic configuration"""
        allow_population_by_field_name = True
        validate_assignment = True

    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.dict(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> BodyUploadUrl:
        """Create an instance of BodyUploadUrl from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self):
        """Returns the dictionary representation of the model using alias"""
        _dict = self.dict(by_alias=True,
                          exclude={
                          },
                          exclude_none=True)
        return _dict

    @classmethod
    def from_dict(cls, obj: dict) -> BodyUploadUrl:
        """Create an instance of BodyUploadUrl from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return BodyUploadUrl.parse_obj(obj)

        _obj = BodyUploadUrl.parse_obj({
            "url": obj.get("url"),
            "chunk_size": obj.get("chunk_size") if obj.get("chunk_size") is not None else 400,
            "chunk_overlap": obj.get("chunk_overlap") if obj.get("chunk_overlap") is not None else 100,
            "summary": obj.get("summary") if obj.get("summary") is not None else False
        })
        return _obj



# flake8: noqa

# import apis into api package
from cheshire_cat_api.api.memory_api import MemoryApi
from cheshire_cat_api.api.plugins_api import PluginsApi
from cheshire_cat_api.api.rabbit_hole_api import RabbitHoleApi
from cheshire_cat_api.api.settings_embedder_api import SettingsEmbedderApi
from cheshire_cat_api.api.settings_general_api import SettingsGeneralApi
from cheshire_cat_api.api.settings_large_language_model_api import SettingsLargeLanguageModelApi
from cheshire_cat_api.api.settings_prompt_api import SettingsPromptApi
from cheshire_cat_api.api.status_api import StatusApi




# coding: utf-8

"""
    😸 Cheshire-Cat API

    Customizable AI architecture  # noqa: E501

    The version of the OpenAPI document: 0.0.5
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""

import re  # noqa: F401
import io
import warnings

from pydantic import validate_arguments, ValidationError
from typing_extensions import Annotated

from pydantic import Field, StrictBool, StrictBytes, StrictInt, StrictStr

from typing import Any, Dict, Optional, Union

from cheshire_cat_api.models.body_upload_url import BodyUploadUrl

from cheshire_cat_api.api_client import ApiClient
from cheshire_cat_api.api_response import ApiResponse
from cheshire_cat_api.exceptions import (  # noqa: F401
    ApiTypeError,
    ApiValueError
)


class RabbitHoleApi(object):
    """NOTE: This class is auto generated by OpenAPI Generator
    Ref: https://openapi-generator.tech

    Do not edit the class manually.
    """

    def __init__(self, api_client=None):
        if api_client is None:
            api_client = ApiClient.get_default()
        self.api_client = api_client

    @validate_arguments
    def upload_file(self, file: Union[StrictBytes, StrictStr], chunk_size: Annotated[Optional[StrictInt], Field(
        description="Maximum length of each chunk after the document is split (in characters)")] = None,
                    chunk_overlap: Annotated[
                        Optional[StrictInt], Field(description="Chunk overlap (in characters)")] = None,
                    summary: Annotated[
                        Optional[StrictBool], Field(description="Enables call to summary hook for this file")] = None,
                    **kwargs) -> object:  # noqa: E501
        """Upload File  # noqa: E501

        Upload a file containing text (.txt, .md, .pdf, etc.). File content will be extracted and segmented into chunks. Chunks will be then vectorized and stored into documents memory.  # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True

        >>> thread = api.upload_file(file, chunk_size, chunk_overlap, summary, async_req=True)
        >>> result = thread.get()

        :param file: (required)
        :type file: bytearray
        :param chunk_size: Maximum length of each chunk after the document is split (in characters)
        :type chunk_size: int
        :param chunk_overlap: Chunk overlap (in characters)
        :type chunk_overlap: int
        :param summary: Enables call to summary hook for this file
        :type summary: bool
        :param async_req: Whether to execute the request asynchronously.
        :type async_req: bool, optional
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :return: Returns the result object.
                 If the method is called asynchronously,
                 returns the request thread.
        :rtype: object
        """
        kwargs['_return_http_data_only'] = True
        if '_preload_content' in kwargs:
            raise ValueError(
                "Error! Please call the upload_file_with_http_info method with `_preload_content` instead and obtain raw data from ApiResponse.raw_data")
        return self.upload_file_with_http_info(file, chunk_size, chunk_overlap, summary, **kwargs)  # noqa: E501

    @validate_arguments
    def upload_file_with_http_info(self, file: Union[StrictBytes, StrictStr], chunk_size: Annotated[
        Optional[StrictInt], Field(
            description="Maximum length of each chunk after the document is split (in characters)")] = None,
                                   chunk_overlap: Annotated[
                                       Optional[StrictInt], Field(description="Chunk overlap (in characters)")] = None,
                                   summary: Annotated[Optional[StrictBool], Field(
                                       description="Enables call to summary hook for this file")] = None,
                                   **kwargs) -> ApiResponse:  # noqa: E501
        """Upload File  # noqa: E501

        Upload a file containing text (.txt, .md, .pdf, etc.). File content will be extracted and segmented into chunks. Chunks will be then vectorized and stored into documents memory.  # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True

        >>> thread = api.upload_file_with_http_info(file, chunk_size, chunk_overlap, summary, async_req=True)
        >>> result = thread.get()

        :param file: (required)
        :type file: bytearray
        :param chunk_size: Maximum length of each chunk after the document is split (in characters)
        :type chunk_size: int
        :param chunk_overlap: Chunk overlap (in characters)
        :type chunk_overlap: int
        :param summary: Enables call to summary hook for this file
        :type summary: bool
        :param async_req: Whether to execute the request asynchronously.
        :type async_req: bool, optional
        :param _preload_content: if False, the ApiResponse.data will
                                 be set to none and raw_data will store the 
                                 HTTP response body without reading/decoding.
                                 Default is True.
        :type _preload_content: bool, optional
        :param _return_http_data_only: response data instead of ApiResponse
                                       object with status code, headers, etc
        :type _return_http_data_only: bool, optional
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the authentication
                              in the spec for a single request.
        :type _request_auth: dict, optional
        :type _content_type: string, optional: force content-type for the request
        :return: Returns the result object.
                 If the method is called asynchronously,
                 returns the request thread.
        :rtype: tuple(object, status_code(int), headers(HTTPHeaderDict))
        """

        _params = locals()

        _all_params = [
            'file',
            'chunk_size',
            'chunk_overlap',
            'summary'
        ]
        _all_params.extend(
            [
                'async_req',
                '_return_http_data_only',
                '_preload_content',
                '_request_timeout',
                '_request_auth',
                '_content_type',
                '_headers'
            ]
        )

        # validate the arguments
        for _key, _val in _params['kwargs'].items():
            if _key not in _all_params:
                raise ApiTypeError(
                    "Got an unexpected keyword argument '%s'"
                    " to method upload_file" % _key
                )
            _params[_key] = _val
        del _params['kwargs']

        _collection_formats = {}

        # process the path parameters
        _path_params = {}

        # process the query parameters
        _query_params = []
        # process the header parameters
        _header_params = dict(_params.get('_headers', {}))
        # process the form parameters
        _form_params = []
        _files = {}
        if _params['file']:
            _files['file'] = _params['file']

        if _params['chunk_size']:
            _form_params.append(('chunk_size', _params['chunk_size']))

        if _params['chunk_overlap']:
            _form_params.append(('chunk_overlap', _params['chunk_overlap']))

        if _params['summary']:
            _form_params.append(('summary', _params['summary']))

        # process the body parameter
        _body_params = None
        # set the HTTP header `Accept`
        _header_params['Accept'] = self.api_client.select_header_accept(
            ['application/json'])  # noqa: E501

        # set the HTTP header `Content-Type`
        _content_types_list = _params.get('_content_type',
                                          self.api_client.select_header_content_type(
                                              ['multipart/form-data']))
        if _content_types_list:
            _header_params['Content-Type'] = _content_types_list

        # authentication setting
        _auth_settings = []  # noqa: E501

        _response_types_map = {
            '200': "object",
            '422': "HTTPValidationError",
        }

        return self.api_client.call_api(
            '/rabbithole/', 'POST',
            _path_params,
            _query_params,
            _header_params,
            body=_body_params,
            post_params=_form_params,
            files=_files,
            response_types_map=_response_types_map,
            auth_settings=_auth_settings,
            async_req=_params.get('async_req'),
            _return_http_data_only=_params.get('_return_http_data_only'),  # noqa: E501
            _preload_content=_params.get('_preload_content', True),
            _request_timeout=_params.get('_request_timeout'),
            collection_formats=_collection_formats,
            _request_auth=_params.get('_request_auth'))

    @validate_arguments
    def upload_memory(self, file: Union[StrictBytes, StrictStr], **kwargs) -> object:  # noqa: E501
        """Upload Memory  # noqa: E501

        Upload a memory json file to the cat memory  # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True

        >>> thread = api.upload_memory(file, async_req=True)
        >>> result = thread.get()

        :param file: (required)
        :type file: bytearray
        :param async_req: Whether to execute the request asynchronously.
        :type async_req: bool, optional
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :return: Returns the result object.
                 If the method is called asynchronously,
                 returns the request thread.
        :rtype: object
        """
        kwargs['_return_http_data_only'] = True
        if '_preload_content' in kwargs:
            raise ValueError(
                "Error! Please call the upload_memory_with_http_info method with `_preload_content` instead and obtain raw data from ApiResponse.raw_data")
        return self.upload_memory_with_http_info(file, **kwargs)  # noqa: E501

    @validate_arguments
    def upload_memory_with_http_info(self, file: Union[StrictBytes, StrictStr], **kwargs) -> ApiResponse:  # noqa: E501
        """Upload Memory  # noqa: E501

        Upload a memory json file to the cat memory  # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True

        >>> thread = api.upload_memory_with_http_info(file, async_req=True)
        >>> result = thread.get()

        :param file: (required)
        :type file: bytearray
        :param async_req: Whether to execute the request asynchronously.
        :type async_req: bool, optional
        :param _preload_content: if False, the ApiResponse.data will
                                 be set to none and raw_data will store the 
                                 HTTP response body without reading/decoding.
                                 Default is True.
        :type _preload_content: bool, optional
        :param _return_http_data_only: response data instead of ApiResponse
                                       object with status code, headers, etc
        :type _return_http_data_only: bool, optional
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the authentication
                              in the spec for a single request.
        :type _request_auth: dict, optional
        :type _content_type: string, optional: force content-type for the request
        :return: Returns the result object.
                 If the method is called asynchronously,
                 returns the request thread.
        :rtype: tuple(object, status_code(int), headers(HTTPHeaderDict))
        """

        _params = locals()

        _all_params = [
            'file'
        ]
        _all_params.extend(
            [
                'async_req',
                '_return_http_data_only',
                '_preload_content',
                '_request_timeout',
                '_request_auth',
                '_content_type',
                '_headers'
            ]
        )

        # validate the arguments
        for _key, _val in _params['kwargs'].items():
            if _key not in _all_params:
                raise ApiTypeError(
                    "Got an unexpected keyword argument '%s'"
                    " to method upload_memory" % _key
                )
            _params[_key] = _val
        del _params['kwargs']

        _collection_formats = {}

        # process the path parameters
        _path_params = {}

        # process the query parameters
        _query_params = []
        # process the header parameters
        _header_params = dict(_params.get('_headers', {}))
        # process the form parameters
        _form_params = []
        _files = {}
        if _params['file']:
            _files['file'] = _params['file']

        # process the body parameter
        _body_params = None
        # set the HTTP header `Accept`
        _header_params['Accept'] = self.api_client.select_header_accept(
            ['application/json'])  # noqa: E501

        # set the HTTP header `Content-Type`
        _content_types_list = _params.get('_content_type',
                                          self.api_client.select_header_content_type(
                                              ['multipart/form-data']))
        if _content_types_list:
            _header_params['Content-Type'] = _content_types_list

        # authentication setting
        _auth_settings = []  # noqa: E501

        _response_types_map = {
            '200': "object",
            '422': "HTTPValidationError",
        }

        return self.api_client.call_api(
            '/rabbithole/memory/', 'POST',
            _path_params,
            _query_params,
            _header_params,
            body=_body_params,
            post_params=_form_params,
            files=_files,
            response_types_map=_response_types_map,
            auth_settings=_auth_settings,
            async_req=_params.get('async_req'),
            _return_http_data_only=_params.get('_return_http_data_only'),  # noqa: E501
            _preload_content=_params.get('_preload_content', True),
            _request_timeout=_params.get('_request_timeout'),
            collection_formats=_collection_formats,
            _request_auth=_params.get('_request_auth'))

    @validate_arguments
    def upload_url(self, body_upload_url: BodyUploadUrl, **kwargs) -> object:  # noqa: E501
        """Upload Url  # noqa: E501

        Upload a url. Website content will be extracted and segmented into chunks. Chunks will be then vectorized and stored into documents memory.  # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True

        >>> thread = api.upload_url(body_upload_url, async_req=True)
        >>> result = thread.get()

        :param body_upload_url: (required)
        :type body_upload_url: BodyUploadUrl
        :param async_req: Whether to execute the request asynchronously.
        :type async_req: bool, optional
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :return: Returns the result object.
                 If the method is called asynchronously,
                 returns the request thread.
        :rtype: object
        """
        kwargs['_return_http_data_only'] = True
        if '_preload_content' in kwargs:
            raise ValueError(
                "Error! Please call the upload_url_with_http_info method with `_preload_content` instead and obtain raw data from ApiResponse.raw_data")
        return self.upload_url_with_http_info(body_upload_url, **kwargs)  # noqa: E501

    @validate_arguments
    def upload_url_with_http_info(self, body_upload_url: BodyUploadUrl, **kwargs) -> ApiResponse:  # noqa: E501
        """Upload Url  # noqa: E501

        Upload a url. Website content will be extracted and segmented into chunks. Chunks will be then vectorized and stored into documents memory.  # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True

        >>> thread = api.upload_url_with_http_info(body_upload_url, async_req=True)
        >>> result = thread.get()

        :param body_upload_url: (required)
        :type body_upload_url: BodyUploadUrl
        :param async_req: Whether to execute the request asynchronously.
        :type async_req: bool, optional
        :param _preload_content: if False, the ApiResponse.data will
                                 be set to none and raw_data will store the 
                                 HTTP response body without reading/decoding.
                                 Default is True.
        :type _preload_content: bool, optional
        :param _return_http_data_only: response data instead of ApiResponse
                                       object with status code, headers, etc
        :type _return_http_data_only: bool, optional
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the authentication
                              in the spec for a single request.
        :type _request_auth: dict, optional
        :type _content_type: string, optional: force content-type for the request
        :return: Returns the result object.
                 If the method is called asynchronously,
                 returns the request thread.
        :rtype: tuple(object, status_code(int), headers(HTTPHeaderDict))
        """

        _params = locals()

        _all_params = [
            'body_upload_url'
        ]
        _all_params.extend(
            [
                'async_req',
                '_return_http_data_only',
                '_preload_content',
                '_request_timeout',
                '_request_auth',
                '_content_type',
                '_headers'
            ]
        )

        # validate the arguments
        for _key, _val in _params['kwargs'].items():
            if _key not in _all_params:
                raise ApiTypeError(
                    "Got an unexpected keyword argument '%s'"
                    " to method upload_url" % _key
                )
            _params[_key] = _val
        del _params['kwargs']

        _collection_formats = {}

        # process the path parameters
        _path_params = {}

        # process the query parameters
        _query_params = []
        # process the header parameters
        _header_params = dict(_params.get('_headers', {}))
        # process the form parameters
        _form_params = []
        _files = {}
        # process the body parameter
        _body_params = None
        if _params['body_upload_url'] is not None:
            _body_params = _params['body_upload_url']

        # set the HTTP header `Accept`
        _header_params['Accept'] = self.api_client.select_header_accept(
            ['application/json'])  # noqa: E501

        # set the HTTP header `Content-Type`
        _content_types_list = _params.get('_content_type',
                                          self.api_client.select_header_content_type(
                                              ['application/json']))
        if _content_types_list:
            _header_params['Content-Type'] = _content_types_list

        # authentication setting
        _auth_settings = []  # noqa: E501

        _response_types_map = {
            '200': "object",
            '422': "HTTPValidationError",
        }

        return self.api_client.call_api(
            '/rabbithole/web/', 'POST',
            _path_params,
            _query_params,
            _header_params,
            body=_body_params,
            post_params=_form_params,
            files=_files,
            response_types_map=_response_types_map,
            auth_settings=_auth_settings,
            async_req=_params.get('async_req'),
            _return_http_data_only=_params.get('_return_http_data_only'),  # noqa: E501
            _preload_content=_params.get('_preload_content', True),
            _request_timeout=_params.get('_request_timeout'),
            collection_formats=_collection_formats,
            _request_auth=_params.get('_request_auth'))
